@@ -43,8 +43,6 @@
 #undef Xrealloc
 #undef Xfree
 
-#define NEED_REPLIES
-#define FONT_PCF
 #include "glxserver.h"
 #include <GL/glxtokens.h>
 #include "g_disptab.h"
@@ -82,7 +80,7 @@ Display *GetBackEndDisplay( __GLXclientState *cl, int s )
    if (! cl->be_displays[s] ) {
       cl->be_displays[s] = XOpenDisplay( DisplayString(dmxScreens[s].beDisplay) );
    }
-   return( cl->be_displays[s] );
+   return cl->be_displays[s];
 }
 
 /*
@@ -630,7 +628,7 @@ int GetCurrentBackEndTag(__GLXclientState *cl, GLXContextTag tag, int s)
       return( cl->be_currentCTag[ (tag-1)*screenInfo.numScreens + s ] );
    }
    else {
-      return( 0 );
+      return 0;
    }
 }
 
@@ -1538,7 +1536,7 @@ int __glXGetVisualConfigs(__GLXclientState *cl, GLbyte *pc)
     int i, p;
 
     screen = req->screen;
-    if (screen > screenInfo.numScreens) {
+    if (screen >= screenInfo.numScreens) {
 	/* The client library must send a valid screen number. */
 	client->errorValue = screen;
 	return BadValue;
@@ -2450,7 +2448,7 @@ int __glXQueryExtensionsString(__GLXclientState *cl, GLbyte *pc)
     len = (int)be_reply.length;
     numbytes = (int)be_reply.n;
     slop = numbytes * __GLX_SIZE_INT8 & 3;
-    be_buf = (char *)Xalloc(numbytes);
+    be_buf = (char *)malloc(numbytes);
     if (!be_buf) {
         /* Throw data on the floor */
         _XEatData(dpy, len);
@@ -2528,7 +2526,7 @@ int __glXQueryServerString(__GLXclientState *cl, GLbyte *pc)
     len = (int)be_reply.length;
     numbytes = (int)be_reply.n;
     slop = numbytes * __GLX_SIZE_INT8 & 3;
-    be_buf = (char *)Xalloc(numbytes);
+    be_buf = (char *)malloc(numbytes);
     if (!be_buf) {
         /* Throw data on the floor */
         _XEatData(dpy, len);
@@ -2687,7 +2685,7 @@ int __glXGetFBConfigs(__GLXclientState *cl, GLbyte *pc)
     int numFBConfigs, i, p;
     __GLXscreenInfo *pGlxScreen;
 
-    if (screen > screenInfo.numScreens) {
+    if (screen >= screenInfo.numScreens) {
 	/* The client library must send a valid screen number. */
 	client->errorValue = screen;
 	return BadValue;
@@ -2843,6 +2841,7 @@ int __glXGetFBConfigs(__GLXclientState *cl, GLbyte *pc)
 
 	if (client->swapped) {
 	    __GLX_DECLARE_SWAP_VARIABLES;
+	    __GLX_DECLARE_SWAP_ARRAY_VARIABLES;
 	    __GLX_SWAP_INT_ARRAY((int *)buf, 2*numAttribs);
 	}
 	WriteToClient(client, 2*numAttribs * __GLX_SIZE_CARD32, (char *)buf);
@@ -2949,7 +2948,7 @@ int __glXCreateWindow(__GLXclientState *cl, GLbyte *pc)
 	return BadAlloc;
     }
 
-    pGlxWindow = (__glXWindow *) xalloc(sizeof(__glXWindow));
+    pGlxWindow = (__glXWindow *) malloc(sizeof(__glXWindow));
     if (!pGlxWindow) {
 	return BadAlloc;
     }
@@ -3018,7 +3017,7 @@ int __glXQueryContext(__GLXclientState *cl, GLbyte *pc)
     reply.n = nProps;
 
     nReplyBytes = reply.length << 2;
-    sendBuf = (int *)xalloc(nReplyBytes);
+    sendBuf = (int *)malloc(nReplyBytes);
     pSendBuf = sendBuf;
     *pSendBuf++ = GLX_FBCONFIG_ID;
     *pSendBuf++ = (int)(ctx->pFBConfig->id);
@@ -3033,7 +3032,7 @@ int __glXQueryContext(__GLXclientState *cl, GLbyte *pc)
         WriteToClient(client, sz_xGLXQueryContextReply, (char *)&reply);
         WriteToClient(client, nReplyBytes, (char *)sendBuf);
     }
-    xfree((char *)sendBuf);
+    free((char *)sendBuf);
 
     return Success;
 }
@@ -3063,7 +3062,7 @@ int __glXQueryContextInfoEXT(__GLXclientState *cl, GLbyte *pc)
     reply.n = nProps;
 
     nReplyBytes = reply.length << 2;
-    sendBuf = (int *)xalloc(nReplyBytes);
+    sendBuf = (int *)malloc(nReplyBytes);
     pSendBuf = sendBuf;
     *pSendBuf++ = GLX_SHARE_CONTEXT_EXT;
     *pSendBuf++ = (int)(ctx->share_id);
@@ -3080,7 +3079,7 @@ int __glXQueryContextInfoEXT(__GLXclientState *cl, GLbyte *pc)
         WriteToClient(client, sz_xGLXQueryContextInfoEXTReply, (char *)&reply);
         WriteToClient(client, nReplyBytes, (char *)sendBuf);
     }
-    xfree((char *)sendBuf);
+    free((char *)sendBuf);
 
     return Success;
 }
@@ -3108,7 +3107,7 @@ int __glXCreatePbuffer(__GLXclientState *cl, GLbyte *pc)
    /*
     ** Look up screen and FBConfig.
     */
-    if (screen > screenInfo.numScreens) {
+    if (screen >= screenInfo.numScreens) {
         /* The client library must send a valid screen number. */
         client->errorValue = screen;
         return BadValue;
@@ -3126,14 +3125,14 @@ int __glXCreatePbuffer(__GLXclientState *cl, GLbyte *pc)
     /*
     ** Create the GLX part of the Pbuffer.
     */
-    pGlxPbuffer = (__glXPbuffer *) xalloc(sizeof(__glXPbuffer));
+    pGlxPbuffer = (__glXPbuffer *) malloc(sizeof(__glXPbuffer));
     if (!pGlxPbuffer) {
         return BadAlloc;
     }
 
-    pGlxPbuffer->be_xids = (XID *) xalloc( sizeof(XID) * screenInfo.numScreens );
+    pGlxPbuffer->be_xids = (XID *) malloc( sizeof(XID) * screenInfo.numScreens );
     if (!pGlxPbuffer->be_xids) {
-        xfree(pGlxPbuffer);
+        free(pGlxPbuffer);
         return BadAlloc;
     }
 
@@ -3382,7 +3381,7 @@ int __glXGetDrawableAttributes(__GLXclientState *cl, GLbyte *pc)
 
    if (reply.numAttribs) {
       attribs_size = 2 * reply.numAttribs * __GLX_SIZE_CARD32;
-      attribs = (CARD32 *) Xalloc(attribs_size);
+      attribs = (CARD32 *) malloc(attribs_size);
       if (attribs == NULL) {
 	 UnlockDisplay(dpy);
 	 SyncHandle();
@@ -3406,7 +3405,7 @@ int __glXGetDrawableAttributes(__GLXclientState *cl, GLbyte *pc)
       WriteToClient(client, attribs_size, (char *)attribs);
    }
 
-   Xfree(attribs);
+   free(attribs);
 
    return Success;
 }
@@ -3422,11 +3421,6 @@ int __glXChangeDrawableAttributes(__GLXclientState *cl, GLbyte *pc)
    Display *dpy;
    int screen, rc;
    DMXScreenInfo *dmxScreen;
-   char *attrbuf;
-#ifdef PANORAMIX
-    PanoramiXRes *pXinDraw = NULL;
-    PanoramiXRes *pXinReadDraw = NULL;
-#endif
 
    if (drawId != None) {
       rc = dixLookupDrawable(&pDraw, drawId, client, 0, DixSetAttrAccess);
@@ -3493,7 +3487,7 @@ int __glXChangeDrawableAttributes(__GLXclientState *cl, GLbyte *pc)
 
 #ifdef PANORAMIX
        if (!noPanoramiXExtension) {
-	  pXinDraw = (PanoramiXRes *)
+	  PanoramiXRes *pXinDraw = (PanoramiXRes *)
 	     SecurityLookupIDByClass(client, pDraw->id, XRC_DRAWABLE, DixReadAccess);
 	  if (!pXinDraw) {
 	     client->errorValue = drawId;
